'use client';
import { jsx } from 'react/jsx-runtime';
import { useState, useEffect, useRef, Children } from 'react';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

/**
 * UseInView
 * Hook using IO to determin if element is visible
 * @example
 *   const ref = useRef<HTMLDivElement>(null)
 *   const isInView = useInView({ ref: ref, rootMargin: '-10px', fireOnce: true })
 * @returns Boolean
 */
function useInView(_a) {
    var ref = _a.ref, _b = _a.rootMargin, rootMargin = _b === void 0 ? '10px' : _b, _c = _a.threshold, threshold = _c === void 0 ? 0.4 : _c, _d = _a.repeat, repeat = _d === void 0 ? false : _d;
    // State and setter for storing whether element is visible
    var _e = useState(false), isIntersecting = _e[0], setIntersecting = _e[1];
    useEffect(function () {
        var callbackFunction = function (entries) {
            var entry = entries[0];
            setIntersecting(entry.isIntersecting);
            if (!repeat && entry.intersectionRatio > 0) {
                observer.unobserve(entry.target);
            }
        };
        var observerRefValue = null;
        var observer = new IntersectionObserver(callbackFunction, {
            rootMargin: rootMargin,
            threshold: threshold
        });
        if (ref.current) {
            observer.observe(ref.current);
            observerRefValue = ref.current;
        }
        return function () {
            if (observerRefValue)
                observer.unobserve(observerRefValue);
        };
    }, [ref, setIntersecting, rootMargin, threshold, repeat]);
    return isIntersecting;
}

var marqueeContainerStyles = {
    position: 'relative',
    width: '100%',
    overflow: 'hidden'
};
var marqueeItemsStyles = function (startPosition, time, direction, willChange) { return (__assign({ display: 'inline-block', whiteSpace: 'nowrap', transform: "translate3d(-".concat(startPosition, "px, 0, 0)"), animationName: 'marqueeScroll', animationDuration: "".concat(time, "s"), animationTimingFunction: 'linear', animationIterationCount: 'infinite', animationPlayState: 'var(--marquee-play)', animationDirection: direction === 'right' ? 'reverse' : undefined }, (willChange && { willChange: 'transform' }))); };
var marqueeItemStyles = function (marginRight) { return ({
    position: 'relative',
    display: 'inline-block',
    marginRight: marginRight
}); };
var getClonedItems = function (items, copyTimes) {
    if (copyTimes === void 0) { copyTimes = 1; }
    return Array(copyTimes).fill(items).flat();
};
var MarqueeText = function (_a) {
    var _b;
    var _c = _a.className, className = _c === void 0 ? 'marquee' : _c, _d = _a.duration, duration = _d === void 0 ? 50 : _d, _e = _a.direction, direction = _e === void 0 ? 'left' : _e, _f = _a.pauseOnHover, pauseOnHover = _f === void 0 ? false : _f, _g = _a.playOnlyInView, playOnlyInView = _g === void 0 ? true : _g, _h = _a.textSpacing, textSpacing = _h === void 0 ? '0.15em' : _h, _j = _a.treshold, treshold = _j === void 0 ? 0.1 : _j, children = _a.children;
    var marqueeContainer = useRef(null);
    var marqueeItems = useRef(null);
    var _k = useState(0), translateFrom = _k[0], setTranslateFrom = _k[1];
    var _l = useState(Children.toArray(children)), showItems = _l[0], setShowItems = _l[1];
    var _m = useState(duration), initialDuration = _m[0], setInitialDuration = _m[1];
    var _o = useState(true), isPlaying = _o[0], setIsPlaying = _o[1];
    var isVisible = useInView({
        ref: marqueeContainer,
        rootMargin: '10px',
        threshold: [treshold],
        repeat: true
    });
    useEffect(function () {
        var containerWidth = Math.floor(marqueeContainer.current.offsetWidth);
        var itemsWidth = Math.floor(marqueeItems.current.scrollWidth);
        var cloneTimes = Math.max(2, Math.ceil((containerWidth * 2) / itemsWidth));
        var translateFromVal = itemsWidth * Math.floor(cloneTimes / 2);
        var durationVal = duration * parseFloat((translateFromVal / containerWidth).toFixed(2));
        setShowItems(getClonedItems(Children.toArray(children), cloneTimes));
        setTranslateFrom(translateFromVal);
        setInitialDuration(durationVal);
    }, [children, duration]);
    useEffect(function () {
        if (!playOnlyInView)
            return;
        if (isVisible) {
            setIsPlaying(true);
        }
        else {
            setIsPlaying(false);
        }
    }, [isVisible, playOnlyInView]);
    var handleMouseEnter = function () {
        if (pauseOnHover) {
            setIsPlaying(false);
        }
    };
    var handleMouseLeave = function () {
        if (pauseOnHover) {
            setIsPlaying(true);
        }
    };
    return (jsx("div", { ref: marqueeContainer, className: "".concat(className), style: __assign(__assign({}, marqueeContainerStyles), (_b = {}, _b['--marquee-play'] = isPlaying ? 'running' : 'paused', _b)), onMouseEnter: handleMouseEnter, onMouseLeave: handleMouseLeave, children: jsx("div", { className: "".concat(className, "__items"), style: marqueeItemsStyles(translateFrom, initialDuration, direction), ref: marqueeItems, children: showItems.map(function (item, index) { return (jsx("div", { className: "".concat(className, "__item"), style: marqueeItemStyles(textSpacing), children: item }, index)); }) }) }));
};

export { MarqueeText as default };
